pub struct DirHandle(pub String);Expand description
A cryptographic handle representing an open directory within the VFS. This acts as a capability token preventing the guest from forging arbitrary paths.
Tuple Fields
0: StringImplementations